@@ -84,7 +84,7 @@ def set():
     result = maxmin()
     print("Max-Min Composition is", result)
     '''
-    return 0
+    return "Package not Installed properly"
 
 def milk():
     '''
@@ -153,7 +153,7 @@ def milk():
 
     print("Best Parameters:", best_params)
     '''
-    return 0
+    return "Package not Installed properly"
 
 
 def art():
@@ -183,7 +183,7 @@ def art():
     plt.show()
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def deap():
     '''
@@ -223,98 +223,116 @@ def deap():
 
     main()
     '''
-    return 0
+    return "Package not Installed properly"
 
 def ant():
     '''
-    import numpy as np
-    from numpy import inf
+import numpy as np
+from numpy import inf
 
-    d = np.array([[0,10,12,11,14],[10,0,13,15,8],[12,13,0,9,14],[11,15,9,0,16],[14,8,14,16,0]])
+#given values for the problems
 
-    iteration = 100
-    n_ants = 5
-    n_citys = 5
-    m = n_ants
-    n = n_citys
-    e = .5        
-    alpha = 1    
-    beta = 2      
-    visibility = 1/d
-    visibility[visibility == inf ] = 0
+d = np.array([[0,10,12,11,14],[10,0,13,15,8],[12,13,0,9,14],[11,15,9,0,16],[14,8,14,16,0]])
 
-    pheromne = .1*np.ones((m,n))
+iteration = 100
+n_ants = 5
+n_citys = 5
 
+# intialization part
+m = n_ants
+n = n_citys
+e = .5         #evaporation rate
+alpha = 1     #pheromone factor
+beta = 2       #visibility factor
 
-    rute = np.ones((m,n+1))
+#calculating the visibility of the next city visibility(i,j)=1/d(i,j)
+visibility = 1/d
+visibility[visibility == inf ] = 0
 
-    for ite in range(iteration):
+#intializing pheromne present at the paths to the cities
+pheromne = .1*np.ones((m,n))
 
-        rute[:,0] = 1          
+#intializing the rute of the ants with size rute(n_ants,n_citys+1)
+#note adding 1 because we want to come back to the source city
 
-        for i in range(m):
+rute = np.ones((m,n+1))
 
-            temp_visibility = np.array(visibility)        
-            for j in range(n-1):
-                #print(rute)
+for ite in range(iteration):
 
-                combine_feature = np.zeros(5)     
-                cum_prob = np.zeros(5)            
+    rute[:,0] = 1          #initial starting and ending positon of every ants '1' i.e city '1'
 
-                cur_loc = int(rute[i,j]-1)       
+    for i in range(m):
 
-                temp_visibility[:,cur_loc] = 0     
+        temp_visibility = np.array(visibility)         #creating a copy of visibility
 
-                p_feature = np.power(pheromne[cur_loc,:],beta)        
-                v_feature = np.power(temp_visibility[cur_loc,:],alpha)  
+        for j in range(n-1):
+            #print(rute)
 
-                p_feature = p_feature[:,np.newaxis]                    
-                v_feature = v_feature[:,np.newaxis]                     
+            combine_feature = np.zeros(5)     #intializing combine_feature array to zero
+            cum_prob = np.zeros(5)            #intializing cummulative probability array to zeros
 
-                combine_feature = np.multiply(p_feature,v_feature)     
+            cur_loc = int(rute[i,j]-1)        #current city of the ant
 
-                total = np.sum(combine_feature)                       
+            temp_visibility[:,cur_loc] = 0     #making visibility of the current city as zero
 
-                probs = combine_feature/total   
-                cum_prob = np.cumsum(probs)     
-                r = np.random.random_sample()  
-                city = np.nonzero(cum_prob>r)[0][0]+1       
+            p_feature = np.power(pheromne[cur_loc,:],beta)         #calculating pheromne feature
+            v_feature = np.power(temp_visibility[cur_loc,:],alpha)  #calculating visibility feature
 
-                rute[i,j+1] = city              
+            p_feature = p_feature[:,np.newaxis]                     #adding axis to make a size[5,1]
+            v_feature = v_feature[:,np.newaxis]                     #adding axis to make a size[5,1]
 
-            left = list(set([i for i in range(1,n+1)])-set(rute[i,:-2]))[0]    
+            combine_feature = np.multiply(p_feature,v_feature)     #calculating the combine feature
 
-            rute[i,-2] = left                   
-        rute_opt = np.array(rute)              
+            total = np.sum(combine_feature)                        #sum of all the feature
 
-        dist_cost = np.zeros((m,1))             
+            probs = combine_feature/total   #finding probability of element probs(i) = comine_feature(i)/total
 
-        for i in range(m):
+            cum_prob = np.cumsum(probs)     #calculating cummulative sum
+            #print(cum_prob)
+            r = np.random.random_sample()   #randon no in [0,1)
+            #print(r)
+            city = np.nonzero(cum_prob>r)[0][0]+1       #finding the next city having probability higher then random(r)
+            #print(city)
 
-            s = 0
-            for j in range(n-1):
+            rute[i,j+1] = city              #adding city to route
 
-                s = s + d[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1]  
-            dist_cost[i]=s                      
+        left = list(set([i for i in range(1,n+1)])-set(rute[i,:-2]))[0]     #finding the last untraversed city to route
 
-        dist_min_loc = np.argmin(dist_cost)             
-        dist_min_cost = dist_cost[dist_min_loc]         
+        rute[i,-2] = left                   #adding untraversed city to route
 
-        best_route = rute[dist_min_loc,:]               
-        pheromne = (1-e)*pheromne                       
+    rute_opt = np.array(rute)               #intializing optimal route
 
-        for i in range(m):
-            for j in range(n-1):
-                dt = 1/dist_cost[i]
-                pheromne[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1] = pheromne[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1] + dt
+    dist_cost = np.zeros((m,1))             #intializing total_distance_of_tour with zero
 
-    print('route of all the ants at the end :')
-    print(rute_opt)
-    print()
-    print('best path :',best_route)
-    print('cost of the best path',int(dist_min_cost[0]) + d[int(best_route[-2])-1,0])
+    for i in range(m):
+
+        s = 0
+        for j in range(n-1):
+
+            s = s + d[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1]   #calcualting total tour distance
+
+        dist_cost[i]=s                      #storing distance of tour for 'i'th ant at location 'i'
+
+    dist_min_loc = np.argmin(dist_cost)             #finding location of minimum of dist_cost
+    dist_min_cost = dist_cost[dist_min_loc]         #finging min of dist_cost
+
+    best_route = rute[dist_min_loc,:]               #intializing current traversed as best route
+    pheromne = (1-e)*pheromne                       #evaporation of pheromne with (1-e)
+
+    for i in range(m):
+        for j in range(n-1):
+            dt = 1/dist_cost[i]
+            pheromne[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1] = pheromne[int(rute_opt[i,j])-1,int(rute_opt[i,j+1])-1] + dt
+            #updating the pheromne with delta_distance
+            #delta_distance will be more with min_dist i.e adding more weight to that route  peromne
+
+print('route of all the ants at the end :')
+print(rute_opt)
+print()
+print('best path :',best_route)
+print('cost of the best path',int(dist_min_cost[0]) + d[int(best_route[-2])-1,0])
     '''
-    return 0
+    return "Package not Installed properly"
 
 
 def rpcc():
@@ -341,7 +359,7 @@ def rpcc():
             print(f"An unexpected error occurred: {e}")
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def rpcs():
     '''
@@ -357,7 +375,7 @@ def rpcs():
     server.serve_forever()
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def rmic():
     '''
@@ -377,7 +395,7 @@ def rmic():
         main()
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def rmis():
     '''
@@ -404,7 +422,7 @@ def rmis():
         main()
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def dload():
     '''
@@ -442,7 +460,7 @@ def dload():
         simulate_dynamic_load_balancing(dynamic_load_balancer, 20)
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 
 def rload():
@@ -477,7 +495,7 @@ def rload():
         simulate_client_requests(load_balancer, 10)
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 def map():
     '''
@@ -521,7 +539,7 @@ def map():
         CalculateGrades.run()
 
     '''
-    return 0
+    return "Package not Installed properly"
 
 cjava = '''
         import java.rmi.Naming;
@@ -606,6 +624,18 @@ cjava = '''
         '''
 
 sjava = '''
+        import java.rmi.Remote;
+        import java.rmi.RemoteException;
+        import java.util.Map;
+
+        public interface HotelInterface extends Remote {
+        boolean bookRoom(String guestName, int roomNumber) throws RemoteException;
+
+        boolean cancelBooking(String guestName) throws RemoteException;
+
+        Map<String, Integer> getBookings() throws RemoteException;
+        }
+
         import java.rmi.RemoteException;
         import java.rmi.server.UnicastRemoteObject;
         import java.util.HashMap;
@@ -673,4 +703,16 @@ sjava = '''
             }
         }
         }
-        '''
\ No newline at end of file
+        '''
+
+def install():
+    """
+    deap==1.4.1
+    sklearn==1.3.2
+    tensorflow_hub==0.16.1
+    tensorflow==2.16.1
+    cv2==4.8.0.76
+    matplotlib==3.8.0
+    pyro5==5.15
+    mrjob==0.7.4"""
+    return "Package not Installed properly"
\ No newline at end of file
